/*
 * @(#)Arrow2D.java
 */

import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;

/**
 * Arrow2D.java - Desenho de uma seta usando Graphics2D.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Esta classe define um objeto para desenhar setas em canvas.
 *
 * <p>
 * O cdigo original foi obtido de 
 * <code>http://www.stat.vt.edu/~sundar/java/code/Arrow.html</code>
 * e modificado.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */

public class Arrow2D
{
  public static final int SIDE_NONE=0,
                          SIDE_LEAD=1,
                          SIDE_TRAIL=2,
                          SIDE_BOTH=3,
                          SIDE_LEAD_LEFT=4,
                          SIDE_LEAD_RIGHT=5,
                          SIDE_TRAIL_LEFT=6,
                          SIDE_TRAIL_RIGHT=7;
                          
  public final double pi=Math.PI;

  protected Line2D.Double line;
  protected Line2D.Double tip;
  
  public Arrow2D()
  { 
   line = new Line2D.Double( );
   tip = new Line2D.Double( );
  }

 /** Draw Arrow */
  public void DrawArrow(Graphics2D g,
                        double xi, double yi,
                        double alfa, double length,
               		int side,
               		double beta, double tip_length) {
	   
    // xi & yi     ->  Initial points of the arrow
    // alfa        ->  Angle of the arrow in relation to the horizontal line
    // length      ->  Length of the arrow
    // side        ->  Side of the arrow
    // beta        ->  Angle of opening of the arrow
    // tip_length  ->  Length of the tip of the arrow
    
    try {
      if (length < 0) { 
        alfa+=pi;
        length*=-1;
      }
      
      double xf;
      double yf;

      xf = xi + length*Math.cos(alfa);
      yf = yi + length*Math.sin(alfa);
      
      line.setLine( xi, yi, xf, yf );
      
      g.draw( line );

      switch (side) {
        case SIDE_NONE :
          break;
        case SIDE_LEAD :
          DrawTipArrow( g, xf, yf, alfa+(3*pi/2.0)-beta, tip_length );
          DrawTipArrow( g, xf, yf, alfa+(pi/2.0)+beta, tip_length );
          break;
        case SIDE_TRAIL :
          DrawTipArrow( g, xi, yi, alfa-beta, tip_length );
          DrawTipArrow( g, xi, yi, alfa+beta, tip_length );
          break;
        case SIDE_BOTH :
          DrawTipArrow( g, xi, yi, alfa-beta, tip_length );
          DrawTipArrow( g, xi, yi, alfa+beta, tip_length );
          DrawTipArrow( g, xf, yf, alfa+(3*pi/2.0)-beta, tip_length );
          DrawTipArrow( g, xf, yf, alfa+(pi/2.0)+beta, tip_length );
          break;
        case SIDE_LEAD_LEFT :
          DrawTipArrow( g, xf, yf, alfa+(pi/2.0)+beta, tip_length );
          break;
        case SIDE_LEAD_RIGHT :
          DrawTipArrow( g, xf, yf, alfa+(3*pi/2.0)-beta, tip_length );
          break;
        case SIDE_TRAIL_LEFT :
          DrawTipArrow( g, xi, yi, alfa+beta, tip_length );
          break;
        case SIDE_TRAIL_RIGHT :
          DrawTipArrow( g, xi, yi, alfa-beta, tip_length );
          break;
        default:
          throw new IllegalArgumentException();
      }
    }
    catch (IllegalArgumentException iae) {
      System.out.println("Invalid value for variable side.");
    }
  }
  
  protected void DrawTipArrow(Graphics2D g,
                              double x1, double y1,
                              double beta, double tip_length)
  {
   double x2;
   double y2;

   x2 = x1 + tip_length*Math.cos(beta);
   y2 = y1 + tip_length*Math.sin(beta);

   tip.setLine( x1, y1, x2, y2 );

   g.draw( tip );
  }
}

