/*
 * @(#)MohrArc.java
 */

import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;

import wc.*;

/**
 * MohrArc.java - Desenho de um arco genrico com nome.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a definio da classe para um arco
 * genrico com nome para desenhar em um canvas.
 *
 * A funo de desenho recebe a seguinte informao para 
 * desenho:
 *
 * - Centro do arco
 * - Ponto que define ngulo inicial
 * - Ponto que define ngulo final
 * - Raio do arco
 * - String com texto do nome
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
public class MohrArc
{

 /*
 ** ---------------------------------------------------------------
 ** Protected instance variables:
 */

 /** Objeto arco */
 protected Arc2D.Double arc;

  
 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

 /*=======================  Constructor  =========================*/
 /**
  * Cria um objeto MohrPlane.
  */
 public MohrArc( )
 {
  arc = new Arc2D.Double( );
 }

/*=============================  getAng  =========================*/
/**
 * Esta funo retorna o valor do ngulo em radianos que a linha 
 * (x-xc,y-yc) faz com a horizontal.
 */
 protected static double getAng( double xc, double yc, double x, double y )
 {
  double   s;
  double   ang;

  s = Math.sqrt( (x-xc) * (x-xc) + (y-yc) * (y-yc) );
  if( s == 0.0 )
   return( Math.PI/2.0 );

  ang = Math.atan2( y-yc, x-xc );

  return( ang );
 }

 /*========================  DrawArc  =========================*/
 /**
  * Desenha um arco de acordo com os dados fornecidos.
  * @param centerx - coordenada x do centro do arco
  * @param centery - coordenada y do centro do arco
  * @param point0x - coordenada x do ponto que define ngulo inicial
  * @param point0y - coordenada y do ponto que define ngulo inicial
  * @param point1x - coordenada x do ponto que define ngulo final
  * @param point1y - coordenada y do ponto que define ngulo final
  * @param radius  - raio do arco
  * @param name    - string com texto do nome
  */
 public void DrawArc( CanvasWC canvas, Graphics2D g, Color c,
                      double centerx, double centery,
                      double point0x, double point0y,
                      double point1x, double point1y,
                      double radius, String name, double shift,
                      BasicStroke arc_stroke )
 {
  double ang0 = getAng( centerx, centery, point0x, point0y );
  double ang1 = getAng( centerx, centery, point1x, point1y );
  double diam = radius * 2.0;
  double angext;
  double angmid;
  double sin_a;
  double cos_a;
  double x;
  double y;
  double pi = Math.PI;

  if( (ang0 > 0.0) && (ang1 < 0.0) )
   angext = 2.0*pi + (ang1 - ang0);
  else if( ang1 < ang0 )
   angext = 2.0*pi + (ang1 - ang0);
  else
   angext = ang1 - ang0;
  angmid = ang0 + (angext * 0.5);

  g.setColor( c );
  g.setStroke( arc_stroke );

  arc.setArc( centerx-radius, centery-radius, diam, diam, 
              -Math.toDegrees( ang0 ), -Math.toDegrees( angext ),
              Arc2D.OPEN );
  g.draw( arc );

  g.setColor( Color.black );
  canvas.setTextAlignment( CanvasWC.CENTER );
  sin_a = Math.sin( angmid );
  cos_a = Math.cos( angmid );
  x = centerx + (radius+shift)*cos_a;
  y = centery + (radius+shift)*sin_a;
  canvas.drawString( name, x, y );
 }

 /*========================  DrawCircle  =========================*/
 /**
  * Desenha um circulo completo de acordo com os dados fornecidos.
  * @param centerx - coordenada x do centro do arco
  * @param centery - coordenada y do centro do arco
  * @param point0x - coordenada x do ponto que define ngulo inicial
  * @param point0y - coordenada y do ponto que define ngulo inicial
  * @param radius  - raio do arco
  * @param name    - string com texto do nome
  */
 public void DrawCircle( CanvasWC canvas, Graphics2D g, Color c,
                         double centerx, double centery,
                         double point0x, double point0y,
                         double radius, String name, double shift,
                         BasicStroke arc_stroke )
 {
  double ang0 = getAng( centerx, centery, point0x, point0y );
  double diam = radius * 2.0;
  double angmid;
  double sin_a;
  double cos_a;
  double x;
  double y;
  double pi = Math.PI;

  angmid = ang0 + pi;

  g.setColor( c );
  g.setStroke( arc_stroke );

  arc.setArc( centerx-radius, centery-radius, diam, diam, 
              -Math.toDegrees( ang0 ), -Math.toDegrees( 2.0*pi ),
              Arc2D.OPEN );
  g.draw( arc );

  g.setColor( Color.black );
  canvas.setTextAlignment( CanvasWC.CENTER );
  sin_a = Math.sin( angmid );
  cos_a = Math.cos( angmid );
  x = centerx + (radius+shift)*cos_a;
  y = centery + (radius+shift)*sin_a;
  canvas.drawString( name, x, y );
 }

}
