/*
 * @(#)MohrCircleCanvas.java
 */

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.text.*;

import wc.*;

/**
 * MohrCircleCanvas.java - Desenha o crculo de Mohr.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a classe que desenha e manipula o 
 * crculo de Mohr.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrCircleCanvas extends CanvasWC
{
 /** Tamanho de referncia para definio de strokes e tamanho de objetos */
 protected double ref_size;

 /** Ajuste de janela mostrando origem ou enquadra no crculo */
 public static final int SHOW_ORIGIN = 0;

 /** Ajuste de janela enquadrando no crculo */
 public static final int FIT_ON_CIRCLE = 1; 

 /** Tipo de ajuste de janela */
 protected int fit_type;

 /** Visibilidade do estado de tenses corrente */
 protected boolean vis_statestrs = true;

 /** Visibilidade das tenses principal */
 protected boolean vis_principalstrs = true;

 /** Visibilidade das tenses no plano inclinado */
 protected boolean vis_thetastrs = true;

 /** Nenhum ponto selecionado */
 public static final int NONE_SEL = 0; 

 /** Ponto do plano theta selecionado */
 public static final int THETA_PLANE_SEL = 1; 

 /** Ponto do sigmax selecionado */
 public static final int SIGMAX_SEL = 2; 

 /** Ponto do sigmay selecionado */
 public static final int SIGMAY_SEL = 3;

 /** Ponto do tauxy selecionado */
 public static final int TAUXY_SEL = 4;

 /** Ponto do tauyx (-tauxy) selecionado */
 public static final int TAUYX_SEL = 5;

 /** tipo de ponto selecionado pelo mouse */
 protected static int type_sel;

 /** Driver do programa Mohr-Circle */
 protected MohrDriver driver;

 /** Crculo  */
 protected Ellipse2D.Double circle;

 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico do crculo */
 protected static final double CIRCLE_STROKE_FAC = 0.003;

 /** Stroke bsico do crculo */
 protected BasicStroke circle_stroke;

 /** Eixo horizontal das tenses normais */
 protected Arrow2D sigaxis;

 /** Coordenada x do incio do eixo horizontal */
 protected double sigaxis_start;

 /** Coordenada x do final do eixo horizontal */
 protected double sigaxis_end;

 /** Comprimento do eixo horizontal */
 protected double sigaxis_len;
 
 /** Eixo vertical das tenses de cisalhamento */
 protected Arrow2D tauaxis;

 /** Comprimento do eixo vertical */
 protected double tauaxis_len;

 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico dos eixos */
 protected static final double AXIS_STROKE_FAC = 0.002;

 /** Stroke bsico dos eixos */
 protected BasicStroke axis_stroke;

 /** Disco do polo */
 protected Ellipse2D.Double pole_disk;
 
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para tamanho de disco */
 protected static final double DISK_SIZE_FAC = 0.01;

 /** Tamanho de um disco */
 protected double disk_size;

 /** Disco do centro */
 protected Ellipse2D.Double center_disk;
 
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para tamanho do disco do centro */
 protected static final double CENTER_DISK_SIZE_FAC = 0.005;

 /** Tamanho do disco do centro */
 protected double center_disk_size;
  
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico das linhas de tenses */
 protected static final double STRSLINE_STROKE_FAC = 0.001;
 
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico do tracejado das 
  *  linhas de tenses */
 protected static final double STRSLINE_STROKE_DASH_FAC = 0.01;
 
 /** Vetor de especificao do espaamento entre desenho opaco e 
  *  transparente do tracejado das linhas de tenso */
 protected float DASHp[];
 
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico do tracejado das 
  *  linhas de tenses correspondentes ao estado de tenso fornecido */
 protected static final double STATESTRSLINE_STROKE_DASH_FAC = 0.02;
 
 /** Vetor de especificao do espaamento entre desenho opaco e 
  *  transparente do tracejado das linhas das tenses correspondentes 
  *  ao estado de tenso fornecido */
 protected float DASHss[];
 
 /** Stroke bsico das linhas de tenses */
 protected BasicStroke strsline_stroke;
 
 /** Stroke bsico das linhas de tenses do plano de tenses fornecido */
 protected BasicStroke statestrsline_stroke;

 /** Objeto plano para desenhar tenses em um plano */
 protected MohrPlane plane;

 /** Objeto arco para desenhar ngulos */
 protected MohrArc arc;

 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico das setas */
 protected static final double STRESS_ARROW_STROKE_FAC = 0.002;

 /** Stroke bsico das setas das tenses */
 protected BasicStroke stress_arrow_stroke;
 
 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico das linhas dos planos */
 protected static final double PLANE_FACE_STROKE_FAC = 0.0015;

 /** Stroke bsico das linhas dos planos */
 protected BasicStroke plane_face_stroke;

 /** Fonte adotado para desenho */
 protected Font fnt;

 /** Objeto linha para desenho de linhas auxiliares */

 protected Line2D.Double auxline;

 /** Fator de proporo em relao ao tamanho horizontal da janela
  *  em coordenadas do mundo para stroke bsico das linhas auxiliares */
 protected static final double AUXLINE_STROKE_FAC = 0.002;

 /** Stroke bsico das linhas auxiliares */
 protected BasicStroke auxline_stroke;

 /** Componente x de um vetor auxiliar */
 protected static double auxvecx;

 /** Componente y de um vetor auxiliar */
 protected static double auxvecy;

 /** Disco da tenso normal x do estado de tenses fornecido */
 protected Ellipse2D.Double sigmax_disk;
 
 /** Disco da tenso normal y do estado de tenses fornecido */
 protected Ellipse2D.Double sigmay_disk;

 /** Armazena a coordenada x corrente do mouse */
 protected double last_x;

 /** Armazena a coordenada y corrente do mouse */
 protected double last_y;
 
 /** Flag para boto do mouse pressionado */
 protected boolean pressedInPoint;

 /** Flag para boto do mouse pressionado */
 protected MohrControls icon;

 /** Centro do crculo de Mohr */
 protected double center;

 /** Rio do crculo de Mohr */
 protected double radius;

 /** Dimetro do crculo de Mohr */
 protected double diameter;

 /** Tenso normal na direo x */
 protected double sigmax;

 /** Tenso normal na direo y */
 protected double sigmay;

 /** Tenso de cisalhamento */
 protected double tauxy;

 /** Tenso principal mxima */
  protected double sigma1;

 /** Tenso principal mnima */
  protected double sigma2;

 /** Angulo do plano tenso principal mxima */
 protected double thetap;

 /** Tenso de cisalhamento mxima */
 protected double taumax;

 /** Coordenada x do plo */
 protected double poleX;

 /** Coordenada y do plo */
 protected double poleY;

 /** Angulo do plano inclinado */
 protected double theta;

 /** Tenso normal no plano inclinado */
 protected double sigtheta;

 /** Tenso de cisalhamento no plano inclinado */
 protected double tautheta;
 

/*==========================  Constructor  =========================*/
/**
  * Constructor: Especifica eixo Y no sentido para cima.
  */
 MohrCircleCanvas( MohrDriver driver )
 {
  super( UPWARD_Y );
  this.driver = driver;

  double fit_space;
  
  fit_type = SHOW_ORIGIN;

  type_sel = NONE_SEL;

  setBackground( Color.white );

  update_sizes( );

  fit_space = sigaxis_len/20.0;

  setWorld( sigaxis_start-fit_space, sigaxis_end+fit_space, 
            -(tauaxis_len+fit_space), tauaxis_len+fit_space );

  ref_size = world_area.width;

  circle = new Ellipse2D.Double( center-radius, -radius, diameter, diameter );

  sigaxis = new Arrow2D( );
  tauaxis = new Arrow2D( );

  circle_stroke = new BasicStroke( 
                               (float)(ref_size*CIRCLE_STROKE_FAC) );
  axis_stroke = new BasicStroke( (float)(ref_size*AXIS_STROKE_FAC) );

  disk_size = ref_size * DISK_SIZE_FAC;
  pole_disk = new Ellipse2D.Double( poleX - (disk_size*0.5),
                                    poleY - (disk_size*0.5),
                                    disk_size, disk_size );

  center_disk_size = ref_size * CENTER_DISK_SIZE_FAC;
  center_disk = new Ellipse2D.Double( center - (center_disk_size*0.5),
                                      0.0    - (center_disk_size*0.5),
                                      center_disk_size, center_disk_size );

  DASHp = new float[]{ (float)(ref_size*STRSLINE_STROKE_DASH_FAC) };
  strsline_stroke = new BasicStroke( (float)(ref_size*STRSLINE_STROKE_FAC), 
                                     BasicStroke.CAP_BUTT, 
                                     BasicStroke.JOIN_MITER, 
                                     10.0f, DASHp, 0.0f );

  DASHss = new float[]{ (float)(ref_size*STATESTRSLINE_STROKE_DASH_FAC) };
  statestrsline_stroke = new BasicStroke( (float)(ref_size*AXIS_STROKE_FAC), 
                                          BasicStroke.CAP_BUTT, 
                                          BasicStroke.JOIN_MITER, 
                                          10.0f, DASHss, 0.0f ); 

  stress_arrow_stroke = new BasicStroke( 
                               (float)(ref_size*STRESS_ARROW_STROKE_FAC),
                                          BasicStroke.CAP_BUTT, 
                                          BasicStroke.JOIN_MITER, 10.0f );
  plane_face_stroke = new BasicStroke( 
                               (float)(ref_size*PLANE_FACE_STROKE_FAC) );
  
  plane = new MohrPlane( );

  arc = new MohrArc( );

  sigmax_disk = new Ellipse2D.Double( sigmax - (disk_size*0.5),
                                      0.0 - (disk_size*0.5),
                                      disk_size, disk_size ); 
  sigmay_disk = new Ellipse2D.Double( sigmay - (disk_size*0.5),
                                      0.0 - (disk_size*0.5),
                                      disk_size, disk_size );

  fnt = new Font( "Times New Roman", Font.ITALIC, 14 );
  
  auxline = new Line2D.Double( );
  auxline_stroke = new BasicStroke( (float)(ref_size*AUXLINE_STROKE_FAC) );
 }

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */

/*==========================  udpate_sizes  ===========================*/
/**
 * Armazena em variaveis auxiliares as componentes de um vetor na 
 * direo e sentido dos dois pontos dados e com tamanho especificado.
 */
 protected static void resize_vector( double x0, double y0, 
                                      double x1, double y1,
                                      double size )
 {
  double v_size = Math.sqrt( (x1-x0)*(x1-x0) + (y1-y0)*(y1-y0) );
  auxvecx = size * (x1-x0)/ v_size;
  auxvecy = size * (y1-y0)/ v_size;
 }

/*==========================  udpate_sizes  ===========================*/
/**
 * Atualiza os tamanhos das variveis de refncia.
 */
 protected void update_sizes( )
 {
  center = driver.getSolver().Center();
  radius = driver.getSolver().Radius();
  diameter = 2.0 * radius;
  sigmax = driver.getSolver().getSigmaX();
  sigmay = driver.getSolver().getSigmaY();
  tauxy  = driver.getSolver().getTauXY();
  sigma1 = driver.getSolver().Sigma1();
  sigma2 = driver.getSolver().Sigma2();
  thetap = driver.getSolver().ThetaP();
  taumax = driver.getSolver().TauMax();
  poleX = driver.getSolver().PoleX();
  poleY = driver.getSolver().PoleY();
  theta = driver.getSolver().getTheta();
  sigtheta = driver.getSolver().SigmaTheta();
  tautheta = driver.getSolver().TauTheta(); 

  if( driver.getSolver().IsHydrostatic() ) 	
  { 
   sigaxis_start = 0.0;
   sigaxis_end = 2.0*center;
   sigaxis_len = sigaxis_end - sigaxis_start;
   tauaxis_len = sigaxis_len * 0.3;
   return;
  }

  if( fit_type == FIT_ON_CIRCLE )
  {
   sigaxis_start = sigma2;
   sigaxis_end = sigma1;
  }
  else
  {
   if( sigma2 > 0.0 )
    sigaxis_start = 0.0;
   else
    sigaxis_start = sigma2;

   if( sigma1 > 0.0 )
    sigaxis_end = sigma1;
   else
    sigaxis_end = 0.0;
  }

  sigaxis_len = sigaxis_end - sigaxis_start;
  if( fit_type == FIT_ON_CIRCLE )
  {
   sigaxis_start -= 0.2*sigaxis_len;
   sigaxis_end += 0.2*sigaxis_len;
  }
  else
  {
   if( sigma2 < 0.0 )
    sigaxis_start -= 0.2*sigaxis_len;
   else
    sigaxis_start = 0.0;
   if( sigma1 > 0.0 )
    sigaxis_end += 0.2*sigaxis_len;
   else
    sigaxis_end = 0.0;
  }
  sigaxis_len = sigaxis_end - sigaxis_start;

  tauaxis_len = 1.2 * radius;
 }

/*=============================  update_geom  ============================*/
/**
  * Atualiza parmetros geomtricos dos objetos do canvas.
  */
 protected void update_geom( )
 {
  circle.setFrame( center-radius, -radius, diameter, diameter );
 
  circle_stroke = new BasicStroke( 
                               (float)(ref_size*CIRCLE_STROKE_FAC) );
  axis_stroke = new BasicStroke( (float)(ref_size*AXIS_STROKE_FAC) );

  disk_size = ref_size * DISK_SIZE_FAC;
  pole_disk.setFrame( poleX - (disk_size*0.5), poleY - (disk_size*0.5),
                      disk_size, disk_size );

  center_disk_size = ref_size * CENTER_DISK_SIZE_FAC;
  center_disk.setFrame( center - (center_disk_size*0.5),
                        0.0    - (center_disk_size*0.5),
                        center_disk_size, center_disk_size );

  DASHp = new float[]{ (float)(ref_size*STRSLINE_STROKE_DASH_FAC) };
  strsline_stroke = new BasicStroke( (float)(ref_size*STRSLINE_STROKE_FAC), 
                                     BasicStroke.CAP_BUTT, 
                                     BasicStroke.JOIN_MITER, 
                                     10.0f, DASHp, 0.0f );

  DASHss = new float[]{ (float)(ref_size*STATESTRSLINE_STROKE_DASH_FAC) };
  statestrsline_stroke = new BasicStroke( (float)(ref_size*AXIS_STROKE_FAC), 
                                          BasicStroke.CAP_BUTT, 
                                          BasicStroke.JOIN_MITER, 
                                          10.0f, DASHss, 0.0f );

  stress_arrow_stroke = new BasicStroke( 
                               (float)(ref_size*STRESS_ARROW_STROKE_FAC),
                                          BasicStroke.CAP_BUTT, 
                                          BasicStroke.JOIN_MITER, 10.0f );
  plane_face_stroke = new BasicStroke( 
                               (float)(ref_size*PLANE_FACE_STROKE_FAC) );

  sigmax_disk.setFrame( sigmax - (disk_size*0.5),
                        0.0 - (disk_size*0.5),
                        disk_size, disk_size ); 
  sigmay_disk.setFrame( sigmay - (disk_size*0.5),
                        0.0 - (disk_size*0.5),
                        disk_size, disk_size );

  auxline_stroke = new BasicStroke( (float)(ref_size*AUXLINE_STROKE_FAC) );
 }

/*================================getTheta==============================*/
/**
 * Esta funo retorna o valor do ngulo que a linha (x-poleX,y-poleY) 
 * faz com a horizontal.
 */  
 protected double getTheta( double x, double y )
 {
  double   s;

  s = Math.sqrt( (x-poleX) * (x-poleX) + (y-poleY) * (y-poleY) );
  if( s < 0.001 )
  {
   if( Math.abs( poleX-center ) < 0.001 )
   {
    if( poleY > 0.0 )
     return( Math.PI );
    else
     return( 0.0 );
   }
   else
   {
    if( poleX > center )
     return( Math.atan2( -poleY, poleX-center ) );
    else
     return( Math.atan2( poleY, center-poleX ) );
   }
  }

  return( Math.atan2( y-poleY, x-poleX ) );
 }

/*===========================setSolverTheta==============================*/
/**
 * Esta funo atualiza o valor do ngulo do plano que gira.
 */  
 protected void setSolverTheta( double angt )
 {
  driver.getSolver().setTheta( angt );
  driver.update_draft( );
 }

/*================================setSigmax==============================*/
/**
 * Esta funo atualiza o valor da tenso sigmax do 
 * estado de tenses corrente.
 */  
 protected void setSigmax( double sigx )
 {
  driver.getSolver().setSigmaX( sigx );
  driver.update_draft( );
 }
 
/*================================setSigmay==============================*/ 
/**
 * Esta funo atualiza o valor da tenso sigmay do 
 * estado de tenses corrente.
 */  
 protected void setSigmay( double sigy )
 {
  driver.getSolver().setSigmaY( sigy );
  driver.update_draft( );
 }

/*=================================setTauxy===============================*/ 
/**
 * Esta funo atualiza o valor da tenso tauxy do 
 * estado de tenses corrente.
 */
 protected void setTauxy( double txy )
 {
  driver.getSolver().setTauXY( txy );
  driver.update_draft( );
 }

/*============================MohrCircleSnapTo============================*/
/**
 * Esta funo atrai um dado ponto para o crculo de Mohr.
 */
 protected static Point2D.Double MohrCircleSnapTo(
                                        double cx, double cy, double r, 
                                        Point2D.Double pt )
 {
  double s,t;
  s = Math.sqrt((pt.x-cx)* (pt.x-cx) + (pt.y-cy)* (pt.y-cy));
  if( s != 0 )
  {
   t = r/s;
   pt.x = cx + t*(pt.x - cx);
   pt.y = cy + t*(pt.y - cy);
  }
  return( pt );
 }
 
/*================================= pickPoint==============================*/
/**
 * Este mtodo retorna verdadeiro para um dos pontos 
 * selecionados e seta o tipo do ponto selecionado.
 */
 protected boolean pickPoint( Point2D.Double pt )
 {
  double tol_fac = 0.08;
  double tol = radius* tol_fac;
  double x_point = sigtheta;
  double y_point = tautheta;
 
  if( (Math.abs(pt.x - sigtheta) <= tol) &&
      (Math.abs(pt.y - tautheta) <= tol) &&
      (vis_thetastrs) )
  {
   type_sel = THETA_PLANE_SEL;
   return (true);
  }
  else if( (Math.abs(pt.x  - sigmax) <= tol) &&
           (Math.abs(pt.y  +  tauxy) <= tol) &&
           (vis_statestrs) )
  {
   type_sel = TAUXY_SEL;
   return(true);
  }
  else if( (Math.abs(pt.x  - sigmay) <= tol) &&
           (Math.abs(pt.y  -  tauxy) <= tol) &&
           (vis_statestrs) )
  {
   type_sel = TAUYX_SEL;
   return(true);
  }
  else if( (Math.abs(pt.x - sigmax) <= tol) &&
           (Math.abs(pt.y -    0.0) <= tol) &&
           (vis_statestrs) )
  {
   type_sel = SIGMAX_SEL;
   return(true);
  }
  else if( (Math.abs(pt.x - sigmay) <= tol) &&
           (Math.abs(pt.y -    0.0) <= tol) &&
           (vis_statestrs) )
  {
   type_sel = SIGMAY_SEL;
   return(true);
  }

  return (false);
 }

/*===============================updatePointPos==============================*/
/**
 * Esta funo atualiza a posio do ponto selecionado de acordo
 * com a posio do mouse.
 */
 protected void updatePointPos( Point2D.Double mouse_pos )
 {
  double X_new;
  double Y_new;
  double X_var;
  double Y_var;
  double sigx;
  double sigy;
  double txy;
   
  if( type_sel == THETA_PLANE_SEL )
  {
   /*Snap to circle*/
   mouse_pos = MohrCircleSnapTo( center, 0.0, radius, mouse_pos );
     
   /* gets the new point position*/
   X_new = mouse_pos.x;
   Y_new = mouse_pos.y;
   
   /*Update the angle theta, sigmatheta and tautheta*/
   setSolverTheta( getTheta( X_new, Y_new ) );
  }
  else if( type_sel == TAUXY_SEL )
  {
   Y_var = -mouse_pos.y + last_y;
   txy = tauxy + Y_var;
   setTauxy( txy );
  }
  else if( type_sel == TAUYX_SEL )
  {
   Y_var = mouse_pos.y - last_y;
   txy = tauxy + Y_var;
   setTauxy( txy );
  }
  else if( type_sel == SIGMAX_SEL )
  {
   X_var = mouse_pos.x - last_x;
   sigx = sigmax + X_var;
   setSigmax( sigx );
  }
  else if( type_sel == SIGMAY_SEL )
  {
   X_var = mouse_pos.x - last_x;
   sigy = sigmay + X_var;
   setSigmay( sigy );
  }
 }

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

/*===============================  update  =============================*/
/**
 * Atualiza o canvas alterando a janela de visualizao.
 */
 public void update( )
 {
  double fit_space;

  update_sizes( );

  fit_space = sigaxis_len/20.0;

  setWorld( sigaxis_start-fit_space, sigaxis_end+fit_space, 
            -(tauaxis_len+fit_space), tauaxis_len+fit_space );

  ref_size = world_area.width;

  update_geom( );
  repaint( );
 }

/*==========================  udpate_draft  ===========================*/
/**
 * Atualiza o canvas sem alterar a janela de visualizao.
 */
 public void update_draft( )
 {
  update_sizes( );
  update_geom( );
  repaint( );
 }

/*===============================  redraw  =============================*/
 /**
  * Redisplay method.
  * This method is called every time the canvas is updated.
  * Client's concrete class must implement it.
  * @param g is the 2D graphics context that the client uses to display
  */
 public void redraw( Graphics2D g )
 {
  double pi = Math.PI;
  double arrow_len = sigaxis_len/20;
  double arrow_factx = arrow_len/10;
  double offset = driver.getSolver().Radius() * 0.15;
  double fact = arrow_len/10.0;
  double theta_radius;
  double theta_shift;
  double theta_p0x;
  double theta_p0y;
  double theta_p1x;
  double theta_p1y;
  
  g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                      RenderingHints.VALUE_ANTIALIAS_ON );

  g.setFont( fnt );

  /* Ajusta tamanho do eixo de tenses de cisalhamento */
  tauaxis_len = (world_area.height*0.5) / 1.1;

  /* Ajusta tamanho do eixos de tenses normais */
  if( sigaxis_end <= 0.0 )
  {
   sigaxis_end += 0.025 * sigaxis_len;
   sigaxis_len = sigaxis_end - sigaxis_start;
  }

  /* Desenha os eixos */
  g.setColor( Color.black );
  g.setStroke( axis_stroke );
  sigaxis.DrawArrow( g, sigaxis_start, 0.0, 0.0, sigaxis_len, 
                     Arrow2D.SIDE_LEAD, 7.0*pi/18.0, tauaxis_len/30.0);
  setTextAlignment( NORTH_EAST );
  drawString( "\u03C3", sigaxis_end+fact, -fact );
  if( (sigaxis_start <= 0.0) && (sigaxis_end >= 0.0) )
  {
   tauaxis.DrawArrow( g, 0.0, 0.0, pi/2, tauaxis_len, Arrow2D.SIDE_LEAD, 
                      7.0*pi/18.0, tauaxis_len/30.0 );
   setTextAlignment( EAST );
   drawString( "\u03C4", -fact, tauaxis_len+fact );
  }

  /* Trata caso especial de estado hidrosttico de tenses */
  if( driver.getSolver().IsHydrostatic() ) 	
  { 
   g.fill( center_disk );
   setTextAlignment( NORTH );
   drawString( "Estado Hidrosttico de Tenses", center, -fact );
   setTextAlignment( BASE_LEFT );
   drawString( "\u03C3x", sigmax+fact, fact );
   setTextAlignment( BASE_RIGHT );
   drawString( "\u03C3y", sigmay-fact, fact );
   return;
  }

  /* Define cor e stroke para desenhar as linhas auxiliares */
  g.setColor( Color.gray.brighter() ); 
  g.setStroke( auxline_stroke );
   
  if( Math.abs( tauxy ) > 0.01 )
  {
   /* Desenha linha referente ao Sigma X */
   auxline.setLine( sigmax, 0.0, sigmax, -tauxy );
   g.draw( auxline );

   /* Desenha linha referente ao Sigma Y */
   auxline.setLine( sigmay, 0.0, sigmay, tauxy );
   g.draw( auxline );

   /* Desenha linha que liga Sigma X a Sigma Y */
   auxline.setLine( sigmay, tauxy, sigmax, -tauxy );
   g.draw( auxline );
  }

  if( vis_principalstrs )
  {   
   /* Desenha linha referente ao Tau max */
   auxline.setLine( center, 0.0, center, taumax );
   g.draw( auxline );

   if( Math.abs( thetap ) > 0.01 )
   {
    /* Desenha angulo referente  tenso principal mxima (thetap) */
    theta_radius = radius*0.25;
    theta_shift = radius*0.05;
    theta_p0x = poleX + theta_radius + theta_shift;
    theta_p0y = -tauxy;
    if( ((poleX > sigmax) && !vis_thetastrs) || 
        (!vis_statestrs && !vis_thetastrs) )
    {
     auxline.setLine( poleX, poleY, theta_p0x, theta_p0y );
     g.draw( auxline );
    }
    if( poleY <= 0.0 )
    {
     theta_p1x = sigma1;
     theta_p1y = 0.0;
    }
    else
    {
     resize_vector( poleX, poleY, sigma1, 0.0, theta_radius + theta_shift );
     theta_p1x = poleX - auxvecx;
     theta_p1y = poleY - auxvecy;
     auxline.setLine( poleX, poleY, theta_p1x, theta_p1y );
     g.draw( auxline );
    }
    arc.DrawArc( this, g, Color.gray.brighter(),
                 poleX, poleY, theta_p0x, theta_p0y, theta_p1x, theta_p1y, 
                 theta_radius, "\u03B8p", theta_shift, auxline_stroke );

    /* Desenha central angulo 2thetap */
    g.setColor( Color.gray.brighter() ); 
    arc.DrawArc( this, g, Color.gray.brighter(),
                 center, 0.0, sigmax, -tauxy, sigma1, 0.0, 
                 theta_radius, "2\u03B8p", theta_shift, auxline_stroke );
   }
  }

  if( vis_thetastrs )
  {
   /* Desenha angulo theta reference ao polo */
   g.setColor( Color.gray.brighter() ); 
   theta_radius = radius*0.35;
   theta_shift = radius*0.05;
   theta_p0x = poleX + theta_radius + theta_shift;
   theta_p0y = -tauxy;
   if( (poleX >= sigmax) || !vis_statestrs )
   {
    if( theta > 0.01 )
    {
     auxline.setLine( poleX, poleY, theta_p0x, theta_p0y );
     g.draw( auxline );
    }
   }
   if( Math.sqrt( (poleX-sigtheta)*(poleX-sigtheta) +
                  (poleY-tautheta)*(poleY-tautheta) ) < 0.001 )
   {
    if( ((Math.abs( poleX-center ) < 0.001) && (poleY > 0.0)) ||
        (poleX > center) )
    {
     resize_vector( 0.0, 0.0, -poleY, poleX-center, theta_radius+theta_shift );
     theta_p1x = poleX + auxvecx;
     theta_p1y = poleY + auxvecy;
    }
    else
    {
     resize_vector( 0.0, 0.0, poleY, center-poleX, theta_radius+theta_shift );
     theta_p1x = poleX + auxvecx;
     theta_p1y = poleY + auxvecy;
     if( theta > 0.01 )
     {
      auxline.setLine( poleX, poleY, theta_p1x, theta_p1y );
      g.draw( auxline );
     }
    }
   }
   else
   {
    resize_vector( poleX, poleY, sigtheta, tautheta, theta_radius+theta_shift );
    if( tautheta > poleY )
    {
     theta_p1x = poleX + auxvecx;
     theta_p1y = poleY + auxvecy;
    }
    else
    {
     theta_p1x = poleX - auxvecx;
     theta_p1y = poleY - auxvecy;
     if( theta > 0.01 )
     {
      auxline.setLine( poleX, poleY, theta_p1x, theta_p1y );
      g.draw( auxline );
     }
    }
   }
   if( theta > 0.01 )
   {
    arc.DrawArc( this, g, Color.gray.brighter(),
                 poleX, poleY, theta_p0x, theta_p0y, theta_p1x, theta_p1y, 
                 theta_radius, "\u03B8", theta_shift, auxline_stroke );
   }

   /* Desenha o angulo central 2theta */
   g.setColor( Color.gray.brighter() ); 
   auxline.setLine( center, 0.0, sigtheta, tautheta );
   g.draw( auxline );
   if( theta > 0.01 )
   {
    if( theta < (pi-0.01) )
     arc.DrawArc( this, g, Color.gray.brighter(),
                  center, 0.0, sigmax, -tauxy, sigtheta, tautheta, 
                  theta_radius, "2\u03B8", theta_shift, auxline_stroke );
    else
     arc.DrawCircle( this, g, Color.gray.brighter(),
                     center, 0.0, sigmax, -tauxy, 
                     theta_radius, "2\u03B8", theta_shift, auxline_stroke );
   }

   /* Desenha linha referente ao Tau theta */
   auxline.setLine( sigtheta, 0.0, sigtheta, tautheta );
   g.setColor( Color.black ); 
   g.draw( auxline );
  }

  /* Desenha o crculo */
  g.setColor( Color.black );
  g.draw( circle );

  /* Desenha o centro */
  g.fill( center_disk );

  if( vis_principalstrs )
  {   
  /* Desenha plano com tenso principal mxima */
   plane.DrawPlane( this, g, Color.red, center, poleX, poleY, 
                    sigma1, 0.0, "\u03C31", " ", 
                    offset, arrow_len, stress_arrow_stroke, strsline_stroke, 
                    plane_face_stroke, disk_size );

  /* Desenha plano com tenso principal mnima */
   plane.DrawPlane( this, g, Color.red.darker(), center, poleX, poleY, 
                    sigma2, 0.0, "\u03C32", " ", 
                    offset, arrow_len, stress_arrow_stroke, strsline_stroke, 
                    plane_face_stroke, disk_size );

  /* Escreve os textos das tenses principais */
   g.setColor( Color.black );
   setTextAlignment( NORTH_WEST );
   drawString( "\u03C31", sigma1+fact, -fact );
   setTextAlignment( NORTH_EAST );
   drawString( "\u03C32", sigma2-fact, -fact );
   setTextAlignment( BASE_LEFT );
   drawString( "\u03C4m", center+fact, taumax/2.0 );
  }

  if( vis_statestrs )
  {   
  /* Desenha tenses no plano com normal na direcao x */
   plane.DrawPlane( this, g, Color.blue.darker(), center, poleX, poleY, 
                    sigmax, -tauxy, "\u03C3x", "\u03C4xy", 
                    offset, arrow_len, stress_arrow_stroke, 
                    statestrsline_stroke, plane_face_stroke,disk_size );

  /* Desenha tenses no plano com normal na direcao y */
   plane.DrawPlane( this, g, Color.blue.brighter(), center, poleX, poleY, 
                    sigmay, tauxy, "\u03C3y", "\u03C4xy", 
                    offset, arrow_len, stress_arrow_stroke, 
                    statestrsline_stroke, plane_face_stroke,disk_size );

  /* Desenha os discos dos pontos manipulveis de sigmax e sigmay */
   g.setColor( Color.blue.darker() );
   g.fill( sigmax_disk );
   g.setColor( Color.blue.brighter() );
   g.fill( sigmay_disk );
   g.setColor( Color.black );
   g.setStroke( plane_face_stroke );
   g.draw( sigmax_disk );
   g.draw( sigmay_disk );
  }

  if( vis_thetastrs )
  {   
  /* Desenha tenses no plano theta */
   plane.DrawPlane( this, g, Color.green.darker(), center, poleX, poleY, 
                    sigtheta, tautheta, "\u03C3\u03B8", "\u03C4\u03B8", 
                    offset, arrow_len, stress_arrow_stroke, strsline_stroke, 
                    plane_face_stroke, disk_size );

  /* Escreve os textos das tenses no plano theta */
   g.setColor( Color.black );
   setTextAlignment( BASE_LEFT );
   drawString( "\u03C3\u03B8", sigtheta+fact, fact );
   setTextAlignment( BASE_RIGHT );
   drawString( "\u03C4\u03B8", sigtheta-fact, tautheta/2.0 );
  }

  /* Desenha o plo */
  if( vis_statestrs || vis_thetastrs || vis_principalstrs )
  {
   g.setColor( Color.black );
   g.fill( pole_disk );
  }

  /* Escreve os textos das tenses do estado corrente */
  setTextAlignment( BASE_LEFT );
  drawString( "\u03C3x", sigmax+fact, fact );
  setTextAlignment( BASE_RIGHT );
  drawString( "\u03C3y", sigmay-fact, fact );
  if (tauxy != 0.0)
  {
   setTextAlignment( BASE_RIGHT );
   drawString( "-\u03C4xy", sigmax-fact, -tauxy/2.0 );
  }
  if (tauxy != 0.0)
  {
   setTextAlignment( BASE_LEFT );
   drawString( "\u03C4xy", sigmay+fact, tauxy/2.0 );
  }
 }


 /**
  * Handles the event of mouse motion in canvas with no
  * button pressed.
  */
 public void mouseMoved (MouseEvent e)
 {
   if( raster_area == null )
    return;
   Point2D.Double mouse_pos = raster2world( e.getX(),e.getY() );
   if( pickPoint( mouse_pos ) ) {
    setCursor( Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR ) );
   }
   else{
    setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
   }
 }

 /**
  * Handles the event of the user pressing down the mouse button.
  */
 public void mousePressedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
   last_x = mouse_pos.x;
   last_y = mouse_pos.y;
   if (pickPoint(mouse_pos))
   {	   
    pressedInPoint = true;
    setCursor( Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR) );
   }
 }

 /**
  * Handles the event of a user dragging the mouse while holding 
  * down the mouse button.
  */
 public void mouseDraggedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
   if( pressedInPoint)
   {
    updatePointPos(mouse_pos);
   }
   last_x = mouse_pos.x;
   last_y = mouse_pos.y;
 }

 /**
  * Handles the event of a user releasing the mouse button.
  */
  public void mouseReleasedWC( MouseEvent e, Point2D.Double mouse_pos )
 {
   if( pressedInPoint )
   {
    updatePointPos( mouse_pos );
    driver.update( );
   }
   pressedInPoint = false;
   type_sel = NONE_SEL;
 }

 /**
  * Especifica tipo de ajuste de janela.
  */
 public void setFitType( int fit_type )
 {
  this.fit_type = fit_type;
 }

 /**
  * Obtem o tipo corrente de ajuste de janela.
  */
 public int getFitType( )
 {
  return( fit_type );
 }

 /**
  * Especifica visibilidade do estado de tenses corrente.
  */
 public void setVisStateStrs( boolean on_off )
 {
  vis_statestrs = on_off;
 }

 /**
  * Especifica visibilidade das tenses principais.
  */
 public void setVisPrincipalStrs( boolean on_off )
 {
  vis_principalstrs = on_off;
 }

 /**
  * Especifica visibilidade das tenses no plano inclinado.
  */
 public void setVisThetaStrs( boolean on_off )
 {
  vis_thetastrs = on_off;
 }
}

