/*
 * @(#)MohrControls.java
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * MohrControls.java - Controles do programa do crculo de Mohr.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo implementa as classes dos objetos de interface 
 * para controle do programa do crculo de Mohr.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 05/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrControls extends JToolBar
{
 protected MohrFrame owner;
 protected MohrPanel target;
 protected MohrAboutBox infodlg;
 protected MohrTheory theorydlg;

 protected ImageIcon i_ptflag;
 protected ImageIcon i_ptflagpress;
 protected ImageIcon i_ptflagroll;

 protected ImageIcon i_ukflag;
 protected ImageIcon i_ukflagpress;
 protected ImageIcon i_ukflagroll;

 protected ImageIcon i_showorigin;
 protected ImageIcon i_showoriginpress;
 protected ImageIcon i_showoriginroll;

 protected ImageIcon i_fitoncircle;
 protected ImageIcon i_fitoncirclepress;
 protected ImageIcon i_fitoncircleroll;

 protected ImageIcon i_state;
 protected ImageIcon i_statepress;
 protected ImageIcon i_stateroll;

 protected ImageIcon i_theta;
 protected ImageIcon i_thetapress;
 protected ImageIcon i_thetaroll;

 protected ImageIcon i_principal;
 protected ImageIcon i_principalpress;
 protected ImageIcon i_principalroll;

 protected ImageIcon i_info;
 protected ImageIcon i_infopress;
 protected ImageIcon i_inforoll;

 protected ImageIcon i_theory;
 protected ImageIcon i_theorypress;
 protected ImageIcon i_theoryroll;

 protected static JCheckBox cb_setportuguese;
 protected static JCheckBox cb_setenglish;

 protected static JCheckBox cb_showorigin;
 protected static JCheckBox cb_fitoncircle;

 protected static JCheckBox cb_statestrs;
 protected static JCheckBox cb_principalstrs;
 protected static JCheckBox cb_thetastrs;

 protected static JCheckBox cb_infodlg;
 protected static JCheckBox cb_theorydlg;


/*=========================  Constructor  ==========================*/

 MohrControls( MohrFrame owner, MohrPanel target )
 {
  this.owner = owner;
  this.target = target;
  setFloatable( false );

  infodlg = new MohrAboutBox( this );
  theorydlg = new MohrTheory( this );

  Insets insets0 = new Insets(0,0,0,0);
  
  i_ptflag 
       = new ImageIcon(getClass().getResource("images/ptflag.gif"));
  i_ptflagpress 
       = new ImageIcon(getClass().getResource("images/ptflagpress.gif"));
  i_ptflagroll 
       = new ImageIcon(getClass().getResource("images/ptflagroll.gif"));
  
  i_ukflag 
       = new ImageIcon(getClass().getResource("images/ukflag.gif"));
  i_ukflagpress 
       = new ImageIcon(getClass().getResource("images/ukflagpress.gif"));
  i_ukflagroll 
       = new ImageIcon(getClass().getResource("images/ukflagroll.gif"));
  
  i_showorigin 
       = new ImageIcon(getClass().getResource("images/showorigin.gif"));
  i_showoriginpress 
       = new ImageIcon(getClass().getResource("images/showoriginpress.gif"));
  i_showoriginroll 
       = new ImageIcon(getClass().getResource("images/showoriginroll.gif"));

  i_fitoncircle 
       = new ImageIcon(getClass().getResource("images/fitoncircle.gif"));
  i_fitoncirclepress 
       = new ImageIcon(getClass().getResource("images/fitoncirclepress.gif"));
  i_fitoncircleroll 
       = new ImageIcon(getClass().getResource("images/fitoncircleroll.gif"));

  i_state 
       = new ImageIcon(getClass().getResource("images/statestrs.gif"));
  i_statepress 
       = new ImageIcon(getClass().getResource("images/statestrspress.gif"));
  i_stateroll 
       = new ImageIcon(getClass().getResource("images/statestrsroll.gif"));

  i_theta 
       = new ImageIcon(getClass().getResource("images/thetastrs.gif"));
  i_thetapress 
       = new ImageIcon(getClass().getResource("images/thetastrspress.gif"));
  i_thetaroll 
       = new ImageIcon(getClass().getResource("images/thetastrsroll.gif"));

  i_principal 
       = new ImageIcon(getClass().getResource("images/principalstrs.gif"));
  i_principalpress 
       = new ImageIcon(getClass().getResource("images/principalstrspress.gif"));
  i_principalroll 
       = new ImageIcon(getClass().getResource("images/principalstrsroll.gif"));

  i_info 
       = new ImageIcon(getClass().getResource("images/info.gif"));
  i_infopress 
       = new ImageIcon(getClass().getResource("images/infopress.gif"));
  i_inforoll 
       = new ImageIcon(getClass().getResource("images/inforoll.gif"));

  i_theory 
       = new ImageIcon(getClass().getResource("images/theory.gif"));
  i_theorypress 
       = new ImageIcon(getClass().getResource("images/theorypress.gif"));
  i_theoryroll 
       = new ImageIcon(getClass().getResource("images/theoryroll.gif"));
  
  cb_setportuguese = new JCheckBox( i_ptflagpress );
  cb_setportuguese.setMargin( insets0 );
  cb_setportuguese.setSelected( true );
  cb_setportuguese.setToolTipText( "Lngua Portuguesa" );
  cb_setportuguese.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    setPortuguese( false ); 
   }
  } );

  cb_setenglish = new JCheckBox( i_ukflag );
  cb_setenglish.setMargin( insets0 );
  cb_setenglish.setSelected( false );
  cb_setenglish.setRolloverIcon( i_ukflagroll );
  cb_setenglish.setToolTipText( "English Language" );
  cb_setenglish.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    setEnglish( false ); 
   }
  } );
  
  cb_showorigin = new JCheckBox( i_showoriginpress );
  cb_showorigin.setMargin( insets0 );
  cb_showorigin.setSelected( true );
  cb_showorigin.setToolTipText( "Mostra origem dos eixos" );
  cb_showorigin.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    setShowOrigin(); 
   }
  } );

  cb_fitoncircle = new JCheckBox( i_fitoncircle );
  cb_fitoncircle.setMargin( insets0 );
  cb_fitoncircle.setSelected( false );
  cb_fitoncircle.setRolloverIcon( i_fitoncircleroll );
  cb_fitoncircle.setToolTipText( "Enquadra no crculo" );
  cb_fitoncircle.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    setFitOnCircle(); 
   }
  } );

  cb_statestrs = new JCheckBox( i_statepress );
  cb_statestrs.setMargin( insets0 );
  cb_statestrs.setSelected( true );
  cb_statestrs.setToolTipText( "Inabilita o estado de tenses x-y" );
  cb_statestrs.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    setStateStrs(); 
   }
  } );

  cb_thetastrs = new JCheckBox( i_thetapress );
  cb_thetastrs.setMargin( insets0 );
  cb_thetastrs.setSelected( true );
  cb_thetastrs.setToolTipText( "Inabilita as tenses no plano theta" );
  cb_thetastrs.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    setThetaStrs(); 
   }
  } );

  cb_principalstrs = new JCheckBox( i_principalpress );
  cb_principalstrs.setMargin( insets0 );
  cb_principalstrs.setSelected( true );
  cb_principalstrs.setToolTipText( "Inabilita as tenses principais" );
  cb_principalstrs.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    setPrincipalStrs(); 
   }
  } );
      
  cb_infodlg = new JCheckBox( i_info );
  cb_infodlg.setMargin( insets0 );
  cb_infodlg.setSelected( false );
  cb_infodlg.setRolloverIcon( i_inforoll );
  cb_infodlg.setToolTipText( "Sobre o programa de crculo de Mohr" );
  cb_infodlg.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    showInfoDlg();
   }
  } );

  cb_theorydlg = new JCheckBox( i_theory );
  cb_theorydlg.setMargin( insets0 );
  cb_theorydlg.setSelected( false );
  cb_theorydlg.setRolloverIcon( i_theoryroll );
  cb_theorydlg.setToolTipText( "Teoria do crculo de Mohr" );
  cb_theorydlg.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    showTheoryDlg();
   }
  } );

  add( cb_setportuguese );
  add( cb_setenglish );
  addSeparator( );
  add( cb_showorigin );
  add( cb_fitoncircle );
  addSeparator( );
  add( cb_statestrs );
  add( cb_thetastrs );
  add( cb_principalstrs );
  addSeparator( );
  add( cb_infodlg );
  add( cb_theorydlg );
 }

/*=============================  setPortuguese  ============================*/

 public void setPortuguese( boolean first_time )
 {
  if( cb_setportuguese.getIcon() != i_ptflagpress )
  {
   cb_setportuguese.setIcon( i_ptflagpress );
   cb_setportuguese.setSelected( true );
   cb_setenglish.setIcon( i_ukflag );
   cb_setenglish.setSelected( false );
   cb_setenglish.setRolloverIcon( i_ukflagroll );

   if( (! first_time) && (owner != null) )
    owner.setPortuguese();
   target.setPortuguese();
   infodlg.setPortuguese();
   theorydlg.setPortuguese();

   cb_setportuguese.setToolTipText( "Lngua Portuguesa" );
   cb_setenglish.setToolTipText( "English Language" );
   cb_showorigin.setToolTipText( "Mostra origem dos eixos" );
   cb_fitoncircle.setToolTipText( "Enquadra no crculo" );
   if( cb_statestrs.getIcon() == i_statepress )
    cb_statestrs.setToolTipText( "Inabilita o estado de tenses x-y" );
   else
    cb_statestrs.setToolTipText( "Habilita o estado de tenses x-y" );
   if( cb_thetastrs.getIcon() == i_thetapress )
    cb_thetastrs.setToolTipText( "Inabilita as tenses no plano inclinado" );
   else
    cb_thetastrs.setToolTipText( "Habilita as tenses no plano inclinado" );
   if( cb_principalstrs.getIcon() == i_principalpress )
    cb_principalstrs.setToolTipText( "Inabilita as tenses principais" );
   else
    cb_principalstrs.setToolTipText( "Habilita as tenses principais" );
   cb_infodlg.setToolTipText( "Sobre o programa de crculo de Mohr" );
   cb_theorydlg.setToolTipText( "Teoria do crculo de Mohr" );
  }
  else
  {
   cb_setportuguese.setSelected( true );
  }
 }

/*=============================  setEnglish  ============================*/

 public void setEnglish( boolean first_time )
 {
  if( cb_setenglish.getIcon() != i_ukflagpress )
  {
   cb_setenglish.setIcon( i_ukflagpress );
   cb_setenglish.setSelected( true );
   cb_setportuguese.setIcon( i_ptflag );
   cb_setportuguese.setSelected( false );
   cb_setportuguese.setRolloverIcon( i_ptflagroll );

   if( (! first_time) && (owner != null) )
    owner.setEnglish();
   target.setEnglish();
   infodlg.setEnglish();
   theorydlg.setEnglish();

   cb_setportuguese.setToolTipText( "Portuguese Language" );
   cb_setenglish.setToolTipText( "English Language" );
   cb_showorigin.setToolTipText( "Show origin of axes" );
   cb_fitoncircle.setToolTipText( "Fit on circle" );
   if( cb_statestrs.getIcon() == i_statepress )
    cb_statestrs.setToolTipText( "Disable x-y stress state" );
   else
    cb_statestrs.setToolTipText( "Enable x-y stress state" );
   if( cb_thetastrs.getIcon() == i_thetapress )
    cb_thetastrs.setToolTipText( "Disable stress on inclined plane" );
   else
    cb_thetastrs.setToolTipText( "Enable stress on inclined plane" );
   if( cb_principalstrs.getIcon() == i_principalpress )
    cb_principalstrs.setToolTipText( "Disable principal stresses" );
   else
    cb_principalstrs.setToolTipText( "Enable principal stresses" );
   cb_infodlg.setToolTipText( "About the Mohr's circle program" );
   cb_theorydlg.setToolTipText( "Theory of Mohr's circle" );
  }
  else
  {
   cb_setenglish.setSelected( true );
  }
 }

/*=============================  setShowOrigin  ============================*/

 protected void setShowOrigin( )
 {
  if( cb_showorigin.getIcon() != i_showoriginpress )
  {
   cb_showorigin.setIcon( i_showoriginpress );
   cb_showorigin.setSelected( true );
   cb_fitoncircle.setIcon( i_fitoncircle );
   cb_fitoncircle.setSelected( false );
   cb_fitoncircle.setRolloverIcon( i_fitoncircleroll );
   target.circle_canvas.setFitType( MohrCircleCanvas.SHOW_ORIGIN );
   target.circle_canvas.update();
  }
  else
  {
   cb_showorigin.setSelected( true );
  }
 }

/*=============================  setFitOnCircle  ============================*/

 protected void setFitOnCircle( )
 {
  if( cb_fitoncircle.getIcon() != i_fitoncirclepress )
  {
   cb_fitoncircle.setIcon( i_fitoncirclepress );
   cb_fitoncircle.setSelected( true );
   cb_showorigin.setIcon( i_showorigin );
   cb_showorigin.setSelected( false );
   cb_showorigin.setRolloverIcon( i_showoriginroll );
   target.circle_canvas.setFitType( MohrCircleCanvas.FIT_ON_CIRCLE );
   target.circle_canvas.update();
  }
  else
  {
   cb_fitoncircle.setSelected( true );
  }
 }

/*=============================  setStateStrs  ============================*/

 protected void setStateStrs( )
 {
  if( cb_statestrs.getIcon() == i_statepress )
  {
   cb_statestrs.setIcon( i_state );
   cb_statestrs.setSelected( false );
   cb_statestrs.setRolloverIcon( i_stateroll );
   if( cb_showorigin.getIcon() == i_ptflagpress )
    cb_statestrs.setToolTipText( "Habilita o estado de tenses x-y" );
   else
    cb_statestrs.setToolTipText( "Enable x-y stress state" );
   target.circle_canvas.setVisStateStrs( false );
  }
  else
  {
   cb_statestrs.setIcon( i_statepress );
   cb_statestrs.setSelected( true );
   if( cb_showorigin.getIcon() == i_ptflagpress )
    cb_statestrs.setToolTipText( "Inabilita o estado de tenses x-y" );
   else
    cb_statestrs.setToolTipText( "Disable x-y stress state" );
   target.circle_canvas.setVisStateStrs( true );
  }
  target.circle_canvas.update();
 }

/*=============================  setThetaStrs  ===========================*/

 protected void setThetaStrs( )
 {
  if( cb_thetastrs.getIcon() == i_thetapress )
  {
   cb_thetastrs.setIcon( i_theta );
   cb_thetastrs.setSelected( false );
   cb_thetastrs.setRolloverIcon( i_thetaroll );
   if( cb_showorigin.getIcon() == i_ptflagpress )
    cb_thetastrs.setToolTipText( "Habilita as tenses no plano inclinado" );
   else
    cb_thetastrs.setToolTipText( "Enable stress on inclined plane" );
   target.circle_canvas.setVisThetaStrs( false );
  }
  else
  {
   cb_thetastrs.setIcon( i_thetapress );
   cb_thetastrs.setSelected( true );
   if( cb_showorigin.getIcon() == i_ptflagpress )
    cb_thetastrs.setToolTipText( "Inabilita as tenses no plano inclinado" );
   else
    cb_thetastrs.setToolTipText( "Disable stress on inclined plane" );
   target.circle_canvas.setVisThetaStrs( true );
  }
  target.circle_canvas.update();
 }
 
/*=============================  setPrincipalStrs  ===========================*/

 protected void setPrincipalStrs( )
 {
  if( cb_principalstrs.getIcon() == i_principalpress )
  {
   cb_principalstrs.setIcon( i_principal );
   cb_principalstrs.setSelected( false );
   cb_principalstrs.setRolloverIcon( i_principalroll );
   if( cb_showorigin.getIcon() == i_ptflagpress )
    cb_principalstrs.setToolTipText( "Habilita as tenses principais" );
   else
    cb_principalstrs.setToolTipText( "Enable principal stresses" );
   target.circle_canvas.setVisPrincipalStrs( false );
  }
  else
  {
   cb_principalstrs.setIcon( i_principalpress );
   cb_principalstrs.setSelected( true );
   if( cb_showorigin.getIcon() == i_ptflagpress )
    cb_principalstrs.setToolTipText( "Inabilita as tenses principais" );
   else
    cb_principalstrs.setToolTipText( "Disable principal stresses" );
   target.circle_canvas.setVisPrincipalStrs( true );
  }
  target.circle_canvas.update();
 }

/*=============================  showInfoDlg  ===========================*/

 protected void showInfoDlg( )
 {
  cb_infodlg.setIcon( i_infopress );
  cb_infodlg.setSelected( true );
  Dimension d1 = infodlg.getSize();
  Dimension d2;
  if( owner != null )
  {
   d2 = owner.getSize();
   int x = Math.max( (d2.width-d1.width)/2, 0 );
   int y = Math.max( (d2.height-d1.height)/2, 0 );
   infodlg.setBounds( x+owner.getX(), y+owner.getY(), d1.width, d1.height );
  }
  else
  {
   infodlg.setBounds( 0, 0, d1.width, d1.height );
  }
  infodlg.show( );
 }

/*=============================  resetInfoDlg  ===========================*/

 void resetInfoDlg( )
 {
  cb_infodlg.setIcon( i_info );
  cb_infodlg.setSelected( false );
  cb_infodlg.setRolloverIcon( i_inforoll );
 }

/*============================  showTheoryDlg  ===========================*/

 protected void showTheoryDlg( )
 {
  cb_theorydlg.setIcon( i_theorypress );
  cb_theorydlg.setSelected( true );
  Dimension d1 = theorydlg.getSize();
  Dimension d2;
  if( owner != null )
  {
   d2 = owner.getSize();
   int x = Math.max( (d2.width-d1.width)/2, 0 );
   int y = Math.max( (d2.height-d1.height)/2, 0 );
   theorydlg.setBounds( x+owner.getX(), y+owner.getY(), d1.width, d1.height );
  }
  else
  {
   theorydlg.setBounds( 0, 0, d1.width, d1.height );
  }
  theorydlg.show( );
 }

/*===========================  resetTheoryDlg  ===========================*/

 void resetTheoryDlg( )
 {
  cb_theorydlg.setIcon( i_theory );
  cb_theorydlg.setSelected( false );
  cb_theorydlg.setRolloverIcon( i_theoryroll );
 }
   
}

