/*
 * @(#)MohrFrame.java
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * MohrFrame.java - Define a janela principal do programa.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Esta classe define a janela principal do programa para 
 * demonstrao do funcionamento do crculo de Mohr para 
 * estado plano de tenses.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
public class MohrFrame extends JFrame
{
 private MohrPanel target;
 private JMenuBar menu_bar;
 private JMenu file_menu;
 private JMenuItem exitoption;
 private MohrDriver driver;
 private boolean is_portuguese;

 MohrFrame( int width, int height, boolean is_portuguese )
 {
  addWindowListener( new WindowAdapter()
  {
   public void windowClosing( WindowEvent e )
   {
    dispose();
    System.exit(0);
   }
  } );
  this.is_portuguese = is_portuguese;
  setTitle( "e-Mohr: crculo de Mohr para estado plano de tenses" );
  target = new MohrPanel( this, is_portuguese );
  buildMenus();
  getContentPane().add( target, BorderLayout.CENTER );
  pack();
  setSize( new Dimension( width, height) );
  if( ! is_portuguese )
   setEnglish();
  show();
 }

 MohrFrame( )
 {
  this( 800, 576, true );
 }

 MohrFrame( int width, int height )
 {
  this( width, height, true );
 }

 MohrFrame( boolean is_portuguese )
 {
  this( 800, 576, is_portuguese );
 }

 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

 /*===========================  setPortuguese  ========================*/

 public void setPortuguese( )
 {
  is_portuguese = true;
  setTitle( "e-Mohr: crculo de Mohr para estado plano de tenses" );
  file_menu.setText( "Arquivo" );
  exitoption.setText( "Sair..." );
 }

/*===========================  setEnglish  ========================*/

 public void setEnglish( )
 {
  is_portuguese = false;
  setTitle( "e-Mohr: Mohr's circle for plane stress state" );
  file_menu.setText( "File" );
  exitoption.setText( "Exit..." );
 }

 /*
 ** ---------------------------------------------------------------
 ** Private and Protected methods: 
 */

 /*===========================  buildMenus  ========================*/
 /**
  * Cria o menu bar.
  */
 private void buildMenus( )
 {
  menu_bar = new JMenuBar();
  file_menu = new JMenu( "Arquivo" );
  addExitItem();
  menu_bar.add( file_menu );
  setJMenuBar( menu_bar );
 }

 /*==========================  addExitItem  ========================*/
 /**
  * Adiciona a opo para sair do programa.
  */
 private void addExitItem()
 {
  exitoption = new JMenuItem( "Sair..." );
  exitoption.setMnemonic('r'); 
  exitoption.addActionListener( new ActionListener()
  {
   public void actionPerformed( ActionEvent e )
   {
    if( exitDialog() == 0 )
    {
     dispose();
     System.exit(0);
    }
   }
  } );
  file_menu.add( exitoption );
 }

 /*==========================  exitDialog  =========================*/
 /**
  * Dialogo para pedir confirmao para sair do programa.
  */
 private int exitDialog()
 {
   int confirm;
   if( is_portuguese )
   {
    confirm = JOptionPane.showOptionDialog( getContentPane(),
                 "Realmente quer sair?", "Confirmao de sada",
                 JOptionPane.YES_NO_OPTION, 
                 JOptionPane.QUESTION_MESSAGE,
                 null, null, null);
   }
   else
   {
    confirm = JOptionPane.showOptionDialog( getContentPane(),
                 "Do you really want to exit?", "Exit Confirmation",
                 JOptionPane.YES_NO_OPTION, 
                 JOptionPane.QUESTION_MESSAGE,
                 null, null, null);
   }
   return( confirm );
 }

}

