/*
 * @(#)MohrPanel.java
 */

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.BorderFactory;

/**
 * MohrPanel.java - Defines the interface panel of the 
 * Mohr-Circle program.
 *
 * <p>
 * Description:
 * <p>
 * ===============================================================
 * <p>
 * This file contains an implementation of the Mohr-Circle
 * for plane stress problems.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 06/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrPanel extends JPanel
{
 /** Resolvedor do crculo de Mohr */
 MohrDriver driver;

 /** Rgua de controles do programa */
 MohrControls controls;

 /** Painel de controle do estado plano de tenses */
 MohrStateStrs statestrs;

 /** Painel de controle do plano para clculo de tenses */
 MohrThetaStrs thetastrs;

 /** Painel de visualizao das tenses principais */
 MohrPrincipalStrs principalstrs;

 /** Canvas do crculo de Mohr */
 MohrCircleCanvas circle_canvas;

 /** Canvas do estado de tenses corrente */
 MohrStateStrsCanvas statestrs_canvas;

 /** Canvas das tenses no plano inclinado */
 MohrThetaStrsCanvas thetastrs_canvas;

 /** Canvas das tenses principais */
 MohrPrincipalStrsCanvas principalstrs_canvas;

/*=========================  Constructors  ==========================*/

 MohrPanel( MohrFrame owner, boolean is_portuguese )
 {
  driver = new MohrDriver( this );
  controls = new MohrControls( owner, this );
  statestrs = new MohrStateStrs( driver );
  thetastrs = new MohrThetaStrs( driver );
  principalstrs = new MohrPrincipalStrs( driver );

  controls.setBorder( BorderFactory.createEtchedBorder() );

  circle_canvas = new MohrCircleCanvas( driver );
  statestrs_canvas = new MohrStateStrsCanvas( driver );
  thetastrs_canvas = new MohrThetaStrsCanvas( driver );
  principalstrs_canvas = new MohrPrincipalStrsCanvas( driver );

  JPanel p_circle= new JPanel();
  p_circle.setLayout( new BorderLayout() );
  p_circle.add( circle_canvas );
  p_circle.setBorder( BorderFactory.createEtchedBorder() );

  JPanel p_statestrs = new JPanel();
  p_statestrs.setLayout( new BorderLayout() );
  p_statestrs.add( "Center", statestrs_canvas );
  p_statestrs.add( "South", statestrs );
  p_statestrs.setBorder( BorderFactory.createEtchedBorder() );

  JPanel p_thetastrs = new JPanel();
  p_thetastrs.setLayout( new BorderLayout() );
  p_thetastrs.add( "Center", thetastrs_canvas );
  p_thetastrs.add( "South", thetastrs );
  p_thetastrs.setBorder( BorderFactory.createEtchedBorder() );

  JPanel p_principalstrs = new JPanel();
  p_principalstrs.setLayout( new BorderLayout() );
  p_principalstrs.add( "Center", principalstrs_canvas );
  p_principalstrs.add( "South", principalstrs );
  p_principalstrs.setBorder( BorderFactory.createEtchedBorder() );

  JPanel p_control = new JPanel();
  p_control.setLayout( new GridLayout( 0, 1 ) );
  p_control.add( p_statestrs );
  p_control.add( p_thetastrs );
  p_control.add( p_principalstrs );

  setLayout( new BorderLayout() );
  add( "North", controls );
  add( "Center", p_circle );
  add( "East", p_control );

  if( ! is_portuguese )
  {
   controls.setEnglish( true );
   setEnglish( );
  }
 }

 MohrPanel( )
 {
  this( null, true );
 }

 MohrPanel( boolean is_portuguese )
 {
  this( null, is_portuguese );
 }

/*=============================  setPortuguese  ============================*/
  
 public void setPortuguese( )
 {
  statestrs.setPortuguese();
  thetastrs.setPortuguese();
  principalstrs.setPortuguese();
 }

/*=============================  setEnglish  ============================*/
  
 public void setEnglish( )
 {
  statestrs.setEnglish();
  thetastrs.setEnglish();
  principalstrs.setEnglish();
 }

}

