/*
 * @(#)MohrPlane.java
 */

import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;

import wc.*;

/**
 * MohrPlane.java - Plano genrico com tenses.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a definio da classe para planos
 * genricos de tenses.
 * <p>
 * ===============================================================
 *
 * @version    1.0 04/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
public class MohrPlane
{

 /*
 ** ---------------------------------------------------------------
 ** Protected instance variables:
 */

 /** Seta para desenhar tenses */
 protected Arrow2D arrow;

 /** Linha do plano */
 protected Line2D.Double plane_line;

 /** Retngulo com corpo do plano */
 protected Rectangle2D.Double plane_body;

 /** Linha da face do plano */
 protected Line2D.Double plane_face;

 /** Transformao afim que faz a rotao em torno do polo */
 protected AffineTransform rot_transf;

 /** Transformao window-viewport corrent */
 protected AffineTransform def_transf;

 /** Disco do plano  */
 protected Ellipse2D.Double plane_disk;

 /** Linha do offset do plano */
 protected Line2D.Double offset_line;
  
 /*
 ** ---------------------------------------------------------------
 ** Public methods: 
 */

 /*=======================  Constructor  =========================*/
 /**
  * Cria um objeto MohrPlane.
  */
 public MohrPlane( )
 {
  arrow = new Arrow2D( );
  plane_line = new Line2D.Double( );
  plane_body = new Rectangle2D.Double( );
  plane_face = new Line2D.Double( );
  rot_transf = new AffineTransform( );
  plane_disk = new Ellipse2D.Double( );
  offset_line = new Line2D.Double( );
 }

 /*========================  DrawPlane  =========================*/
 /**
  * Desenha as tenses no plano de acordo com os dados fornecidos.
  */
 public void DrawPlane( CanvasWC canvas, Graphics2D g, Color c,
                        double center,
                        double polex, double poley,
                        double sig, double tau,
                        String s_sig, String s_tau,
                        double offset, double arrow_len,
                        BasicStroke arrow_stroke, BasicStroke line_stroke,
                        BasicStroke face_stroke, double disk_size )
 {
  double len;
  double radius;
  double sin_a;
  double cos_a;
  double x;
  double y;
  double shift;
  double pi = Math.PI;
  double fact = arrow_len/8.0;
  len = Math.sqrt( (sig-polex)*(sig-polex) + (tau-poley)*(tau-poley) );

  if( len < 0.001 )
  {
   radius = Math.sqrt( (polex-center)*(polex-center) + (poley*poley) );
   cos_a = -poley / radius;
   sin_a = (polex-center) / radius;
  }
  else
  {
   sin_a = (tau-poley) / len;
   cos_a = (sig-polex) / len;
  }

  rot_transf.setTransform( cos_a, sin_a, -sin_a, cos_a, 0.0, 0.0 );

  def_transf = g.getTransform();
  AffineTransform curr_transf = g.getTransform();

  curr_transf.translate( polex, poley );
  curr_transf.concatenate( rot_transf );
  curr_transf.translate( -polex, -poley );
  g.setTransform( curr_transf );

  if( len < (4.0*offset) )
   shift = 4.0 * offset;
  else
   shift = len;

  g.setColor( Color.gray.brighter() );
  g.setStroke( face_stroke );
  offset_line.setLine( polex+len, poley, polex+shift+offset, poley );
  g.draw( offset_line );

  g.setColor( c );
  g.setStroke( arrow_stroke );

  x = polex + shift + offset;
  y = poley;
  plane_body.setFrame( x, y - (arrow_len/2.0), (2.0*fact), arrow_len );
  g.fill( plane_body );
 
  if( Math.abs( sig ) > 0.01 )
  {
   x = polex + shift + offset + (3.0*fact);
   y = poley;
   if( sig > 0.0 )
    arrow.DrawArrow( g, x, y, 0.0, arrow_len/2.0, Arrow2D.SIDE_LEAD, 
                     pi/4.0, arrow_len/6.0 );
   else if( sig < 0.0 )
    arrow.DrawArrow( g, x, y, 0.0, arrow_len/2.0, Arrow2D.SIDE_TRAIL, 
                    pi/4.0, arrow_len/6.0 );
  }

  if( Math.abs( tau ) > 0.01 )
  {
   x = polex + shift + offset + (2.0*fact) + (fact/1.5);
   y = poley - arrow_len/2.0;
   if( tau > 0.0 ) 
    arrow.DrawArrow( g, x, y, pi/2, arrow_len, Arrow2D.SIDE_TRAIL_RIGHT, 
                     pi/4.0, arrow_len/6.0 );
   else if ( tau < 0.0 )
    arrow.DrawArrow( g, x, y, pi/2, arrow_len, Arrow2D.SIDE_LEAD_RIGHT, 
                     pi/4.0, arrow_len/6.0 );
  }

  g.setColor( Color.black );
  g.setStroke( face_stroke );
  canvas.setTextAlignment( CanvasWC.CENTER );

  x = polex + shift + offset + (2.0*fact);
  y = poley;
  plane_face.setLine( x, y - (arrow_len/2.0), x, y + (arrow_len/2.0) );
  g.draw( plane_face );

  if( Math.abs( sig ) > 0.01 )
  {
   x = polex + shift + offset + (arrow_len/2.0) + (5.5*fact);
   y = poley;
   canvas.drawString( s_sig, x, y );
  }

  if( Math.abs( tau ) > 0.01 )
  {
   if( tau > 0 )
   {
    x = polex + shift + offset + (4.5*fact);
    y = poley - (arrow_len/2.0) - (1.5*fact);
    canvas.drawString( s_tau, x, y );
   }
   else if( tau < 0 )
   {
    x = polex + shift + offset + (4.5*fact);
    y = poley + (arrow_len/2.0) + (1.5*fact);
    canvas.drawString( s_tau, x, y );
   }
  }

  g.setTransform( def_transf );

  g.setColor( c );
  g.setStroke( line_stroke );

  plane_line.setLine( polex, poley, sig, tau );
  g.draw( plane_line );
  plane_disk.setFrame( sig - (disk_size*0.5), tau - (disk_size*0.5),
                      disk_size, disk_size );
  g.fill(plane_disk);
   
  g.setColor( Color.black );
  g.setStroke( face_stroke );
  plane_disk.setFrame( sig - (disk_size*0.5), tau - (disk_size*0.5),
                       disk_size, disk_size);
  g.draw(plane_disk);
 }

}
