/*
 * @(#)MohrPrincipalStrs.java
 */

import java.awt.GridLayout;
import java.awt.Dimension;
import java.text.*;
import java.awt.Font;
import java.awt.event.*;
import javax.swing.JPanel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * MohrPrincipalStrs.java - Define o painel com os objetos 
 * texto com os valores das componentes de tenses principais.
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a implementao do objetos JTextField para
 * visualizao dos valores das componentes de tenses principais,
 * direo do plano da tenso principal mxima e valor da tenso
 * de cisalhamento mxima.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 05/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrPrincipalStrs extends JPanel
{
 /** Driver do programa Mohr-Circle */
 protected MohrDriver driver;

 protected ImageIcon i_sig1;
 protected ImageIcon i_sig2;
 protected ImageIcon i_thetap;
 protected ImageIcon i_taumax;

 protected JTextField f_sig1;
 protected JTextField f_sig2;
 protected JTextField f_thetap;
 protected JTextField f_taumax;

 protected DecimalFormat frm_str;
 protected DecimalFormat frm_ang;


 MohrPrincipalStrs( MohrDriver driver )
 {
  this.driver = driver;

  Dimension d_label = new Dimension( 30, 20 );
  frm_str = new DecimalFormat( "####0.00" );
  frm_ang = new DecimalFormat( "###0.0" );
  Font fnt = new Font( "Times New Roman", Font.BOLD | Font.ITALIC, 14 );

  f_sig1 = new JTextField( 5 );
  f_sig1.setEnabled( false );
  f_sig1.setHorizontalAlignment( JTextField.RIGHT );
  f_sig1.setToolTipText( "Tenso principal mxima" );
  JPanel p_sig1 = new JPanel();
  i_sig1 = new ImageIcon( getClass().getResource("images/label_sig1.gif") );
  JLabel l_sig1 = new JLabel( i_sig1 );
  l_sig1.setFont( fnt );
  l_sig1.setPreferredSize( d_label );
  l_sig1.setHorizontalAlignment( JLabel.RIGHT );
  l_sig1.setLabelFor( f_sig1 );
  p_sig1.add( l_sig1 );
  p_sig1.add( f_sig1 );

  f_sig2 = new JTextField( 5 );
  f_sig2.setEnabled( false );
  f_sig2.setHorizontalAlignment( JTextField.RIGHT );
  f_sig2.setToolTipText( "Tenso principal mnima" );
  JPanel p_sig2 = new JPanel();
  i_sig2 = new ImageIcon( getClass().getResource("images/label_sig2.gif") );
  JLabel l_sig2 = new JLabel( i_sig2 );
  l_sig2.setFont( fnt );
  l_sig2.setPreferredSize( d_label );
  l_sig2.setHorizontalAlignment( JLabel.RIGHT );
  l_sig2.setLabelFor( f_sig2 );
  p_sig2.add( l_sig2 );
  p_sig2.add( f_sig2 );

  f_thetap = new JTextField( 5 );
  f_thetap.setEnabled( false );
  f_thetap.setHorizontalAlignment( JTextField.RIGHT );
  f_thetap.setToolTipText( "ngulo do plano de tenso principal" );
  JPanel p_thetap = new JPanel();
  i_thetap = new ImageIcon( getClass().getResource("images/label_thetap.gif") );
  JLabel l_thetap = new JLabel( i_thetap );
  l_thetap.setFont( fnt );
  l_thetap.setHorizontalAlignment( JLabel.RIGHT );
  l_thetap.setPreferredSize( d_label );
  l_thetap.setLabelFor( f_thetap );
  p_thetap.add( l_thetap );
  p_thetap.add( f_thetap );

  f_taumax = new JTextField( 5 );
  f_taumax.setEnabled( false );
  f_taumax.setHorizontalAlignment( JTextField.RIGHT );
  f_taumax.setToolTipText( "Tenso de cisalhamento mxima" );
  JPanel p_taumax = new JPanel();
  i_taumax = new ImageIcon( getClass().getResource("images/label_taumax.gif") );
  JLabel l_taumax = new JLabel( i_taumax );
  l_taumax.setFont( fnt );
  l_taumax.setPreferredSize( d_label );
  l_taumax.setHorizontalAlignment( JLabel.RIGHT );
  l_taumax.setLabelFor( f_taumax );
  p_taumax.add( l_taumax );
  p_taumax.add( f_taumax );

  setLayout( new GridLayout( 0, 2 ) );
  add( p_sig1 );
  add( p_sig2 );
  add( p_thetap );
  add( p_taumax );

  update( );
 }

 public void setPortuguese( )
 {
  f_sig1.setToolTipText( "Tenso principal mxima" );
  f_sig2.setToolTipText( "Tenso principal mnima" );
  f_thetap.setToolTipText( "ngulo do plano de tenso principal" );
  f_taumax.setToolTipText( "Tenso de cisalhamento mxima" );
 }

 public void setEnglish( )
 {
  f_sig1.setToolTipText( "Maximum principal stress" );
  f_sig2.setToolTipText( "Minimum principal stress" );
  f_thetap.setToolTipText( "Angle of principal stress plane" );
  f_taumax.setToolTipText( "Maximum shear stress" );
 }

 protected void updateSigma1( )
 {
  double sig1 = driver.getSolver().Sigma1( );
  f_sig1.setText( frm_str.format( sig1 ) );
 }

 protected void updateSigma2( )
 {
  double sig2 = driver.getSolver().Sigma2( );
  f_sig2.setText( frm_str.format( sig2 ) );
 }

 protected void updateThetaP( )
 {
  double thetap = driver.getSolver().ThetaP( );
  f_thetap.setText( frm_ang.format( Math.toDegrees( thetap ) ) + "" );
 }

 protected void updateTauMax( )
 {
  double taumax = driver.getSolver().TauMax( );
  f_taumax.setText( frm_str.format( taumax ) );
 }

 void update( )
 {
  updateSigma1( );
  updateSigma2( );
  updateThetaP( );
  updateTauMax( );
 }
}

