/*
 * @(#)MohrStateStrs.java
 */

import java.awt.GridLayout;
import java.awt.Dimension;
import java.text.*;
import java.awt.Font;
import java.awt.event.*;
import javax.swing.JPanel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * MohrStateStrs.java - Define o painel com os objetos 
 * texto com os valores das componentes do estado de tenso. 
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a implementao do objetos JTextField para
 * definio e visualizao dos valores do componentes de tenso 
 * do estado corrente.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 05/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrStateStrs extends JPanel
{
 /** Driver do programa Mohr-Circle */
 protected MohrDriver driver;

 protected ImageIcon i_sigx;
 protected ImageIcon i_sigy;
 protected ImageIcon i_tauxy;

 protected JTextField f_sigx;
 protected JTextField f_sigy;
 protected JTextField f_tauxy;

 protected DecimalFormat frm_str;
 protected DecimalFormat frm_ang;


 MohrStateStrs( MohrDriver driver )
 {
  this.driver = driver;

  Dimension d_label = new Dimension( 30, 20 );
  frm_str = new DecimalFormat( "####0.00" );
  frm_ang = new DecimalFormat( "###0.0" );
  Font fnt = new Font( "Times New Roman", Font.BOLD | Font.ITALIC, 14 );

  f_sigx = new JTextField( 5 );
  f_sigx.setHorizontalAlignment( JTextField.RIGHT );
  f_sigx.setToolTipText( "Tenso normal na direo horizontal" );
  JPanel p_sigx = new JPanel();
  i_sigx = new ImageIcon( getClass().getResource("images/label_sigx.gif") );
  JLabel l_sigx = new JLabel( i_sigx );
  l_sigx.setFont( fnt );
  l_sigx.setPreferredSize( d_label );
  l_sigx.setHorizontalAlignment( JLabel.RIGHT );
  l_sigx.setLabelFor( f_sigx );
  p_sigx.add( l_sigx );
  p_sigx.add( f_sigx );
  f_sigx.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent ev) {
     double sigx;
     try {
      sigx = frm_str.parse( f_sigx.getText() ).doubleValue();
     }
     catch( ParseException e ) {
      updateSigmaX( );
      return;
     }
     setSolverSigmaX( sigx );
    }
  });
 
  f_sigy = new JTextField( 5 );
  f_sigy.setHorizontalAlignment( JTextField.RIGHT );
  f_sigy.setToolTipText( "Tenso normal na direo vertical" );
  JPanel p_sigy = new JPanel();
  i_sigy = new ImageIcon( getClass().getResource("images/label_sigy.gif") );
  JLabel l_sigy = new JLabel( i_sigy );
  l_sigy.setFont( fnt );
  l_sigy.setPreferredSize( d_label );
  l_sigy.setHorizontalAlignment( JLabel.RIGHT );
  l_sigy.setLabelFor( f_sigy );
  p_sigy.add( l_sigy );
  p_sigy.add( f_sigy );
  f_sigy.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent ev) {
     double sigy;
     try {
      sigy = frm_str.parse( f_sigy.getText() ).doubleValue();
     }
     catch( ParseException e ) {
      updateSigmaY( );
      return;
     }
     setSolverSigmaY( sigy );
    }
  });

  f_tauxy = new JTextField( 5 );
  f_tauxy.setHorizontalAlignment( JTextField.RIGHT );
  f_tauxy.setToolTipText( "Tenso de cisalhamento para o elemento horizontal" );
  JPanel p_tauxy = new JPanel();
  i_tauxy = new ImageIcon( getClass().getResource("images/label_tauxy.gif") );
  JLabel l_tauxy = new JLabel( i_tauxy );
  l_tauxy.setFont( fnt );
  l_tauxy.setHorizontalAlignment( JLabel.RIGHT );
  l_tauxy.setPreferredSize( d_label );
  l_tauxy.setLabelFor( f_tauxy );
  p_tauxy.add( l_tauxy );
  p_tauxy.add( f_tauxy );
  f_tauxy.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent ev) {
     double tauxy;
     try {
      tauxy = frm_str.parse( f_tauxy.getText() ).doubleValue();
     }
     catch( ParseException e ) {
      updateTauXY( );
      return;
     }
     setSolverTauXY( tauxy );
    }
  });

  setLayout( new GridLayout( 0, 2 ) );
  add( p_sigx );
  add( p_sigy );
  add( p_tauxy );

  update( );
 }

 public void setPortuguese( )
 {
  f_sigx.setToolTipText( "Tenso normal na direo horizontal" );
  f_sigy.setToolTipText( "Tenso normal na direo vertical" );
  f_tauxy.setToolTipText( "Tenso de cisalhamento para o elemento horizontal" );
 }

 public void setEnglish( )
 {
  f_sigx.setToolTipText( "Normal stress in the horizontal direction" );
  f_sigy.setToolTipText( "Normal stress in the vertical direction" );
  f_tauxy.setToolTipText( "Shear stress for the horizontal element" );
 }

 protected void setSolverSigmaX( double sigx )
 {
  driver.getSolver().setSigmaX( sigx );
  driver.update( );
 }

 protected void setSolverSigmaY( double sigy )
 {
  driver.getSolver().setSigmaY( sigy );
  driver.update( );
 }

 protected void setSolverTauXY( double tauxy )
 {
  driver.getSolver().setTauXY( tauxy );
  driver.update( );
 }

 protected void updateSigmaX( )
 {
  double sigx = driver.getSolver().getSigmaX( );
  f_sigx.setText( frm_str.format( sigx ) );
 }

 protected void updateSigmaY( )
 {
  double sigy = driver.getSolver().getSigmaY( );
  f_sigy.setText( frm_str.format( sigy ) );
 }

 protected void updateTauXY( )
 {
  double tauxy = driver.getSolver().getTauXY( );
  f_tauxy.setText( frm_str.format( tauxy ) );
 }

 void update( )
 {
  updateSigmaX( );
  updateSigmaY( );
  updateTauXY( );
 }
}

