/*
 * @(#)MohrThetaStrs.java
 */

import java.awt.GridLayout;
import java.awt.Dimension;
import java.text.*;
import java.awt.Font;
import java.awt.event.*;
import javax.swing.JPanel;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JSlider;
import javax.swing.event.*;

/**
 * MohrThetaStrs.java - Define o painel com os objetos 
 * texto com os valores das componentes de tenso no 
 * plano definido pelo ngulo theta. 
 *
 * <p>
 * Descrio:
 * <p>
 * ===============================================================
 * <p>
 * Este arquivo contm a implementao do objetos JTextField para
 * definio e visualizao dos valores do componentes de tenso 
 * no plano definido pelo ngulo theta.
 *
 * <p>
 * ===============================================================
 *
 * @version    1.0 05/09/2004
 * @author     Luiz Fernando Martha
 * @author     Alonso Juvinao Carbono
 * @author     Anderson Resende Pereira
 * @author     Fernando Busato Ramires
 * @author     Pala Reginal Dalcanal
 * @author     Ricardo Rodrigues de Araujo  
 *
 */
class MohrThetaStrs extends JPanel
{
 /** Driver do programa Mohr-Circle */
 protected MohrDriver driver;

 protected ImageIcon i_theta;
 protected ImageIcon i_sigtheta;
 protected ImageIcon i_tautheta;

 protected JTextField f_theta;
 protected JTextField f_sigtheta;
 protected JTextField f_tautheta;
 protected JSlider    s_theta;

 protected DecimalFormat frm_str;
 protected DecimalFormat frm_ang;

 protected SliderListener s_listener;


 MohrThetaStrs( MohrDriver driver )
 {
	 
  this.driver = driver;

  Dimension d_label = new Dimension( 30, 20 );
  Dimension d_slider = new Dimension( 30, 20 );
  frm_str = new DecimalFormat( "####0.00" );
  frm_ang = new DecimalFormat( "###0.0" );
  Font fnt = new Font( "Times New Roman", Font.BOLD | Font.ITALIC, 14 );

  s_listener = new SliderListener();

  f_theta = new JTextField( 5 );
  f_theta.setHorizontalAlignment( JTextField.RIGHT );
  f_theta.setToolTipText( "ngulo do plano inclinado" );
  JPanel p_theta = new JPanel();
  i_theta = new ImageIcon( getClass().getResource("images/label_theta.gif") );
  JLabel l_theta = new JLabel( i_theta );
  l_theta.setFont( fnt );
  l_theta.setHorizontalAlignment( JLabel.RIGHT );
  l_theta.setPreferredSize( d_label );
  l_theta.setLabelFor( f_theta );
  p_theta.add( l_theta );
  p_theta.add( f_theta );
  f_theta.addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent ev) {
     double theta;
     try {
      theta = frm_ang.parse( f_theta.getText() ).doubleValue();
     }
     catch( ParseException e ) {
      updateTheta( );
      return;
     }
     setSolverTheta( theta );
    }
  });

  f_sigtheta = new JTextField( 5 );
  f_sigtheta.setEnabled( false );
  f_sigtheta.setHorizontalAlignment( JTextField.RIGHT );
  f_sigtheta.setToolTipText( "Tenso normal do plano inclinado" );
  JPanel p_sigtheta = new JPanel();
  i_sigtheta = 
          new ImageIcon( getClass().getResource("images/label_sigtheta.gif") );
  JLabel l_sigtheta = new JLabel( i_sigtheta );
  l_sigtheta.setFont( fnt );
  l_sigtheta.setPreferredSize( d_label );
  l_sigtheta.setHorizontalAlignment( JLabel.RIGHT );
  l_sigtheta.setLabelFor( f_sigtheta );
  p_sigtheta.add( l_sigtheta );
  p_sigtheta.add( f_sigtheta );

  f_tautheta = new JTextField( 5 );
  f_tautheta.setEnabled( false );
  f_tautheta.setHorizontalAlignment( JTextField.RIGHT );
  f_tautheta.setToolTipText( "Tenso de cisalhamento do plano inclinado" );
  JPanel p_tautheta = new JPanel();
  i_tautheta = 
          new ImageIcon( getClass().getResource("images/label_tautheta.gif") );
  JLabel l_tautheta = new JLabel( i_tautheta );
  l_tautheta.setFont( fnt );
  l_tautheta.setPreferredSize( d_label );
  l_tautheta.setHorizontalAlignment( JLabel.RIGHT );
  l_tautheta.setLabelFor( f_tautheta );
  p_tautheta.add( l_tautheta );
  p_tautheta.add( f_tautheta );

  s_theta = new JSlider( 0, 1800, 
                 (int)(Math.toDegrees( driver.getSolver().getTheta( ) )*10.0) );
  s_theta.setToolTipText( "Ajuste do ngulo do plano inclinado" );
  s_theta.setPreferredSize( d_slider );  
  s_theta.addChangeListener( s_listener );
  
  setLayout( new GridLayout( 2, 0 ) );
  add( p_theta );
  add( s_theta );
  add( p_sigtheta );
  add( p_tautheta );

  update( );
 }

 public void setPortuguese( )
 {
  f_theta.setToolTipText( "ngulo do plano inclinado" );
  f_sigtheta.setToolTipText( "Tenso normal do plano inclinado" );
  f_tautheta.setToolTipText( "Tenso de cisalhamento do plano inclinado" );
  s_theta.setToolTipText( "Ajuste do ngulo do plano inclinado" );
 }

 public void setEnglish( )
 {
  f_theta.setToolTipText( "Angle of inclined plane" );
  f_sigtheta.setToolTipText( "Normal stress of inclined plane" );
  f_tautheta.setToolTipText( "Shear stress of inclined plane" );
  s_theta.setToolTipText( "Adjust the angle of inclined plane" );
 }

 protected void setSolverTheta( double theta )
 {
  theta = Math.toRadians( theta );
  driver.getSolver().setTheta( theta );
  driver.update( );
 }

 protected void updateTheta( )
 {
  double theta = driver.getSolver().getTheta( );
  f_theta.setText( frm_ang.format( Math.toDegrees( theta ) ) + "" );
 }

 protected void updateSigmaTheta( )
 {
  double sigtheta = driver.getSolver().SigmaTheta( );
  f_sigtheta.setText( frm_str.format( sigtheta ) );
 }
 
 protected void updateTauTheta( )
 {
  double tautheta = driver.getSolver().TauTheta( );
  f_tautheta.setText( frm_str.format( tautheta ) );
 }

 protected void updateSlider( )
 {
  int val = (int)(Math.toDegrees( driver.getSolver().getTheta( ) )*10.0);
  s_theta.setValue( val );
 }

 void update( )
 {
  updateTheta( );
  updateSlider( );
  updateSigmaTheta( );
  updateTauTheta( );
 }

 class SliderListener implements ChangeListener
 { 
   public void stateChanged(ChangeEvent e) 
   {
    setSolverTheta( s_theta.getValue()/10.0 );
    updateTheta( );
    updateSigmaTheta( );
    updateTauTheta( );
   }
 }

}

